#
# Attempt to package nvidia-container-runtime in Fedora
#
# https://nvidia.github.io/nvidia-container-runtime/
# https://gitlab.com/nvidia/container-toolkit/container-toolkit

%global __golang_extldflags -Wl,-z,lazy

%global goipath             github.com/NVIDIA/nvidia-container-toolkit
%global commit              3ea02d13fc62dcc2d106554bce1fd0fadf5743d6
%global dgx_selinux_commit  b5ae74df51d9ec013b058d9b472df5ea231fc374
%global selinuxtype         targeted
%global container_selinux_v 2.213
%gometa -f


%global _docdir_fmt         %{name}

%global golicenses          LICENSE
%global godocs              README.md DEVELOPMENT.md CONTRIBUTING.md

%global common_description %{expand:
The NVIDIA Container Toolkit allows users to build and run GPU accelerated 
containers. The toolkit includes a container runtime library and utilities 
to automatically configure containers to leverage NVIDIA GPUs.
}


Name:             nvidia-container-toolkit
Version:          0
Release:          %autorelease
Summary:          NVIDIA Container Toolkit
License:          Apache-2.0
URL:              %{gourl}
Source0:          %{gosource}
Source1:          https://github.com/NVIDIA/dgx-selinux/archive/%{dgx_selinux_commit}/dgx-selinux-%{dgx_selinux_commit}.tar.gz
Source2:          nvidia-container-toolkit-selinux.README.txt
Recommends:       (%{name}-selinux = %{version}-%{release} if selinux-policy-%{selinuxtype})
Recommends:       runc
BuildRequires:    selinux-policy-devel, container-selinux >= %{container_selinux_v}

%description
%wordwrap -v common_description


%package selinux
Summary:          NVIDIA Container Toolkit SELinux Policy
License:          MIT
Requires:         %{name} = %{version}-%{release}
BuildArch:        noarch
Requires:         selinux-policy 
Requires:         (container-selinux >= %{container_selinux_v} if selinux-policy-%{selinuxtype})
Requires(post):   policycoreutils
Requires(postun): policycoreutils

%description selinux
%wordwrap -v common_description

SELinux policy to enable the toolkit to use the GPU.

Defines nvidia_container_t to be added for containers 
needing access to the GPU.


%gopkg


%prep
%goprep
tar xf %{SOURCE1}


%generate_buildrequires
%go_generate_buildrequires


%build
for cmd in cmd/* ; do
  %gobuild -o %{gobuilddir}/bin/$(basename $cmd) %{goipath}/$cmd
done

pushd dgx-selinux-%{dgx_selinux_commit}/src/nvidia-container-selinux
#make nvidia-container.pp
touch nvidia-container.pp
popd


%install
install -m 0755 -vd %{buildroot}%{_bindir}
install -m 0755 -vp %{gobuilddir}/bin/nvidia-* %{buildroot}%{_bindir}/

install -m 0755 -vd %{buildroot}%{_libexecdir}/oci/hooks.d
install -m 0755 -t %{buildroot}%{_libexecdir}/oci/hooks.d oci-nvidia-hook
install -m 0755 -vd %{buildroot}%{_datarootdir}/containers/oci/hooks.d
install -m 0644 -t %{buildroot}%{_datarootdir}/containers/oci/hooks.d oci-nvidia-hook.json

install -m 0644 %{SOURCE2} .
install -m 0644 dgx-selinux-%{dgx_selinux_commit}/LICENSE LICENSE-dgx-selinux
install -m 0644 dgx-selinux-%{dgx_selinux_commit}/src/nvidia-container-selinux/README.md dgx-selinux-README.md

pushd dgx-selinux-%{dgx_selinux_commit}/src/nvidia-container-selinux
install -m 0755 -vd %{buildroot}%{_datadir}/selinux/packages/%{selinuxtype}/
install -m 0644 -t %{buildroot}%{_datadir}/selinux/packages/%{selinuxtype} nvidia-container.pp
popd

%gopkginstall


%if %{with check}
%check
%gocheck
%endif


%post selinux
# FIXME: Test current state and don't manage if already set
setsebool -P container_use_devices 1
%selinux_modules_install -s %{selinuxtype} %{_datadir}/selinux/packages/%{selinuxtype}/nvidia-container.pp


%postun selinux
if [ $1 -eq 0 ]; then
# FIXME: If was already set, don't unset
setsebool -P container_use_devices 0
%selinux_modules_uninstall -s %{selinuxtype} nvidia-container
fi


%files
%{_bindir}/nvidia-*
%{_libexecdir}/oci/hooks.d/oci-nvidia-hook
%{_datarootdir}/containers/oci/hooks.d/oci-nvidia-hook.json
%gopkgfiles


%files selinux
%license LICENSE-dgx-selinux
%doc nvidia-container-toolkit-selinux.README.txt dgx-selinux-README.md
%{_datadir}/selinux/packages/%{selinuxtype}/nvidia-container.pp


%changelog
* Mon May 15 2023 Jonathan Steffan <jsteffan@fedoraproject.org> - 0-1
- Initial package hacking
